/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2007 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Scan_Service.h"
#include "Debug.h"
#include "Reseau.h"
#include "Info.h"
#include "Macro.h"
#include <stdlib.h>

void Recupere_Nom_DNS (struct Struct_Liste_Equipement *Equip)
{
	ERROR_if (Equip->Adresse_Ip_Char == NULL);
	PROTECT_CODE (Semaphore_DNS);
	
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Recupere_Nom_DNS = [%s]",Equip->Adresse_Ip_Char);
	Equip->host = NULL;
	Equip->addr = inet_addr ((char *) Equip->Adresse_Ip_Char);
	Equip->host = gethostbyaddr((char *)&Equip->addr, sizeof(struct in_addr),AF_INET);
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin  taille [%d] herr : %d   Erreur : %d",Equip->Taille_Buffer,Equip->herr,Equip->erreur);
	
	if (Equip->host == NULL) {
		switch (h_errno) {
		    case HOST_NOT_FOUND:
			    debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "L'hôte indiqué est inconnu.(%s) [%s]", hstrerror (Equip->herr), Equip->Adresse_Ip_Char);
			    break;
		    case NO_ADDRESS:
			    debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Le nom est valide mais ne possède pas d'adresse IP.(%s) [%s]", hstrerror (Equip->herr), Equip->Adresse_Ip_Char);
			    break;
		    case NO_RECOVERY:
			    debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Une erreur fatale du serveur de noms est apparue.(%s) [%s]", hstrerror (Equip->herr), Equip->Adresse_Ip_Char);
			    break;
		    case TRY_AGAIN:
			    debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Aucune connection à  internet.(%s) [%s]", hstrerror (Equip->herr), Equip->Adresse_Ip_Char);
			    break;
		    default:
			    debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur Inconnue(%d) pour [%s]", Equip->herr, Equip->Adresse_Ip_Char);
			    break;
		}
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Dns Failed.[%s]",Equip->Adresse_Ip_Char);
		Equip->M->Nom_DNS[0] = 0 ;
	}
	else {
		Equip->Taille_Chaine = strlen (Equip->host->h_name);
		if (Equip->Taille_Chaine != 0) {
			if (Equip->Taille_Chaine > TAILLE_CHAINE_NOM_DNS)
				Equip->Taille_Chaine = TAILLE_CHAINE_NOM_DNS;
			strncpy (Equip->M->Nom_DNS, Equip->host->h_name, Equip->Taille_Chaine);
			Equip->M->Nom_DNS[Equip->Taille_Chaine] = 0;
			debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nom DNS = [%s]", Equip->M->Nom_DNS);
		}
	}
	DEPROTECT_CODE (Semaphore_DNS);
}


void Envoie_Secur_Requete(struct Struct_Liste_Equipement *Equip,gchar *Requete,gint Taille_Requete,gchar *Protocol,gchar *Reponse){
	ERROR_if (Equip == NULL);
	ERROR_if (Equip->Requete == NULL);
	ERROR_if (Protocol == NULL);
	Equip->Tentative = 0;

	do { 
		Equip->Tentative++;
		debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s] Socket (%d) tentative (%d)",Protocol,Equip->IdSocket,Equip->Tentative);
		Equip->NbOctet = send (Equip->IdSocket, Requete, Taille_Requete , 0);
		if (Equip->NbOctet < 1){
			debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s] NbOctet == %d   %s  Socket (%d)", Protocol, Equip->NbOctet,strerror (errno),Equip->IdSocket);
			if (Equip->NbOctet == -1){ 
				if (!(errno == EINTR || errno == EAGAIN)){
					if (close_control (Equip->IdSocket) < 0)
						debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s] %s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", Protocol, strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
					Reponse = g_strdup_printf ("Error %s : send NbOctet == -1",Protocol);
					return;
				}
				sleep(1);
			}
			else
				sleep(1);
		}
		else{
			if (Equip->NbOctet != Taille_Requete){
				debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s] NbOctet != Taille_Requete [%s]  Socket (%d)", Protocol, strerror (errno),Equip->IdSocket);
				if (close_control (Equip->IdSocket) < 0)
					debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s] [%s]\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", Protocol, strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
				Reponse = g_strdup_printf ("Error %s : NbOctet != Taille_Requete",Protocol);
				return;
			}
		}
	}while((Equip->Tentative<10) && (Equip->NbOctet == -1) && (errno == EAGAIN || errno == EINTR));
	if (Equip->NbOctet == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s] %s Socket (%d)",Protocol , strerror (errno),Equip->IdSocket);
		if (close_control (Equip->IdSocket) < 0)
			debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s] %s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", Protocol, strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
		return g_strdup_printf("Error %s :  NbOctet < 0   Tentative(%d)",Protocol,Equip->Tentative);
	}
	Nouveau_Octet_Envoye (Equip->NbOctet);
	Reponse = NULL;
	return;
}

char     *Recupere_Banniere_FTP (struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_FTP  "FTP : "
	gint Nb_Error_Consecutive;
	
	usleep (50000);	
	ERROR_if (Equip == NULL);
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion au serveur FTP [%s]", Equip->Adresse_Ip_Char);
	if ((Equip->IdSocket = Connection_Equipement (Equip->Adresse_Ip_Char,NULL, PORT_FTP, Time_Out_Connect_Service, 0,FALSE)) == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion refusée au serveur FTP [%s]", Equip->Adresse_Ip_Char);
		return g_strdup ("Error FTP: Connection refused");
	}

	Equip->chaine = NULL;
	Equip->banniere = NULL;
	Equip->Finale = NULL;
	Equip->Finale2 = NULL;
	Equip->retour_envoie = NULL;
	Equip->nbLigne = 0;
	Equip->fin = FALSE;
	Equip->Tentative = 0;
	Nb_Error_Consecutive = 0;

	strcpy (Equip->Requete, "Bad command\n");
	Envoie_Secur_Requete(Equip,Equip->Requete,strlen(Equip->Requete),"FTP",Equip->retour_envoie);
	if (Equip->retour_envoie!=NULL)
		return Equip->retour_envoie; 

	Equip->TailleBanniere = sizeof (BANNIERE_FTP);
	MALLOC(Equip->banniere,sizeof (char) * Equip->TailleBanniere);
	ERROR_if (Equip->banniere == NULL);
	strcpy (Equip->banniere, BANNIERE_FTP);

	while ((!Equip->fin) && (Equip->Tentative!=50) && (Nb_Error_Consecutive<3) && (attendre_Donnee (Equip->IdSocket, Time_Out_Service))) {
		Equip->NbOctet = recvfrom (Equip->IdSocket, &Equip->Requete, (sizeof(char)*TAILLE_REQUETE_SERVICE) -1, 0, NULL, 0);
		if (Equip->NbOctet < 1 ){
			debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "NbOctet %d  %s  Socket (%d) Erreur Consécutive (%d)", Equip->NbOctet, strerror (errno),Equip->IdSocket,Nb_Error_Consecutive);
			if (Equip->NbOctet == -1){ 
				if (!(errno == EINTR || errno == EAGAIN)){
					if (close_control (Equip->IdSocket) < 0)
						debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
					MYFREE(Equip->banniere); 
					return g_strdup_printf ("Error FTP :  NbOctet == -1");
				}
				sleep(1);
			}
			else
				sleep(1);
			Nb_Error_Consecutive++;
		}
		else {
			Nouveau_Octet_Recu (Equip->NbOctet);
			Equip->Requete[Equip->NbOctet] = 0; 
		
		
			Equip->i = 0;//Equip->NbOctet;
			while ((Equip->i<Equip->NbOctet) && (Equip->Requete[Equip->i]!=0))
				Equip->i++;			
			if (Equip->i!=Equip->NbOctet)
				debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Caractère final %d %d",Equip->NbOctet,Equip->i);
		
			if ((Equip->Finale = strstr(Equip->Requete,"\r\n"))==NULL){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Repere pas trouvé : ajout %d Octect",Equip->NbOctet);
					REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + Equip->NbOctet)));
					strcat (Equip->banniere, Equip->Requete);
					Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
			}
			else {
				if ((Equip->Finale2 = strstr(Equip->Finale+2,"\r\n"))!=NULL){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "2 lignes en un coups");
					Equip->nbLigne++;
					Equip->Finale = Equip->Finale2;
				}
				Equip->nbLigne++;
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Repere trouvé : lignes %d",Equip->nbLigne);
				Equip->NbOctet = Equip->Finale - Equip->Requete + 2;
				REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + Equip->NbOctet)));
				Equip->Requete[Equip->NbOctet] = 0;
				strcat (Equip->banniere, Equip->Requete);
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Bannière OK : %s\n",Equip->banniere);
				Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
				if (Equip->nbLigne==2){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin des donnees a recuperer (FTP)");
					Equip->fin = TRUE;
				}
			}
			Nb_Error_Consecutive = 0;
		}
		Equip->Tentative++;
	}
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Bannière recuperer : %s\n",Equip->banniere);
	if (Equip->TailleBanniere != (strlen (Equip->banniere) + 1))
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur TailleBanniere (%d)!=(%d)",Equip->TailleBanniere,(strlen (Equip->banniere) + 1));
	if (close_control (Equip->IdSocket) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\n[%s] Socket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", Equip->Adresse_Ip_Char,strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	return Equip->banniere;
}


char     *Recupere_Banniere_HTTP (struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_HTTP  "HTTP : "
	gint Nb_Error_Consecutive;
	
	usleep (50000);	
	ERROR_if (Equip == NULL);
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion au serveur HTTP [%s]", Equip->Adresse_Ip_Char);
	if ((Equip->IdSocket = Connection_Equipement (Equip->Adresse_Ip_Char, NULL, PORT_HTTP, Time_Out_Connect_Service, 0,FALSE)) == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion refusée au serveur HTTP [%S]", Equip->Adresse_Ip_Char);
		return g_strdup ("Error HTTP: Connection refused");
	}

	Equip->chaine = NULL;
	Equip->banniere = NULL;
	Equip->Finale = NULL;
	Equip->Finale2 = NULL;
	Equip->retour_envoie = NULL;
	Equip->nbLigne = 0;
	Equip->fin = FALSE;
	Equip->Tentative = 0;
	Nb_Error_Consecutive = 0;

	sprintf (Equip->Requete, "GET / HTTP/1.1\r\nHost: %s\r\n\r\n", Equip->Adresse_Ip_Char);
	Envoie_Secur_Requete(Equip,Equip->Requete,strlen(Equip->Requete),"HTTP",Equip->retour_envoie);
	if (Equip->retour_envoie!=NULL)
		return Equip->retour_envoie;

	Equip->TailleBanniere = sizeof (BANNIERE_HTTP);
	MALLOC(Equip->banniere,sizeof (char) * Equip->TailleBanniere);
	ERROR_if (Equip->banniere == NULL);
	strcpy (Equip->banniere, BANNIERE_HTTP);

	while ((!Equip->fin) && (Equip->Tentative!=50) && (Nb_Error_Consecutive<3) && (attendre_Donnee (Equip->IdSocket, Time_Out_Service))) {
		Equip->NbOctet = recvfrom (Equip->IdSocket, &Equip->Requete, (sizeof(char)*TAILLE_REQUETE_SERVICE) - 1, 0, NULL, 0);
		if (Equip->NbOctet < 1){
			debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "NbOctet == %d   %s  Socket (%d) Erreur Consécutive (%d)",Equip->NbOctet, strerror (errno),Equip->IdSocket,Nb_Error_Consecutive);
			if (Equip->NbOctet == -1){ 
				if (!(errno == EINTR || errno == EAGAIN)){
					if (close_control (Equip->IdSocket) < 0)
						debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
					MYFREE(Equip->banniere); 
					return g_strdup_printf ("Error HTTP :  NbOctet == -1");
				}
				sleep(1);
			}
			else
				sleep(1);
			Nb_Error_Consecutive++;
		}
		else {
			Nouveau_Octet_Recu (Equip->NbOctet);
			Equip->Requete[Equip->NbOctet] = 0; 
		
		
			Equip->i = 0;//Equip->NbOctet;
			while ((Equip->i<Equip->NbOctet) && (Equip->Requete[Equip->i]!=0))
				Equip->i++;			
			if (Equip->i!=Equip->NbOctet)
				debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Caractère final %d %d",Equip->NbOctet,Equip->i);
		
			if ((Equip->Finale = strstr(Equip->Requete,"\r\n\r\n"))==NULL){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Repere pas trouvé  : ajout %d Octect  fin(%d)",Equip->NbOctet,Equip->i);
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s",Equip->Requete);
					REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + Equip->NbOctet)));
					strcat (Equip->banniere, Equip->Requete);
					Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
			}
			else {
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "NbOctet = %d   fin(%d)",Equip->NbOctet,Equip->i);
				Equip->NbOctet = Equip->Finale - Equip->Requete + 2;
				Equip->Requete[Equip->NbOctet] = 0;
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Repere trouvé--  [%s]",Equip->Requete);
				REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + Equip->NbOctet)));
				strcat (Equip->banniere, Equip->Requete);
				Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
				Equip->fin = TRUE;
			}
			Nb_Error_Consecutive = 0;
		}
		Equip->Tentative++;
	}
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Bannière recuperer (%d) : [%s]",Equip->TailleBanniere,Equip->banniere);
	if (Equip->TailleBanniere != (strlen (Equip->banniere) + 1))
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur TailleBanniere (%d)!=(%d)",Equip->TailleBanniere,(strlen (Equip->banniere) + 1));
	if (close_control (Equip->IdSocket) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\n[%s] Socket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", Equip->Adresse_Ip_Char,strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	return Equip->banniere;
}


char     *Recupere_Banniere_IMAP (struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_IMAP  "IMAP : "
	gint Nb_Error_Consecutive;
	
	usleep (50000);	
	ERROR_if (Equip == NULL);
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion au serveur IMAP [%s]", Equip->Adresse_Ip_Char);
	if ((Equip->IdSocket = Connection_Equipement (Equip->Adresse_Ip_Char,NULL, PORT_IMAP, Time_Out_Connect_Service, 0,FALSE)) == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion refusée au serveur IMAP [%s]", Equip->Adresse_Ip_Char);
		return g_strdup ("Error IMAP: Connection refused");
	}

	Equip->chaine = NULL;
	Equip->banniere = NULL;
	Equip->Finale = NULL;
	Equip->Finale2 = NULL;
	Equip->retour_envoie = NULL;
	Equip->nbLigne = 0;
	Equip->fin = FALSE;
	Equip->Tentative = 0;
	Nb_Error_Consecutive = 0;

	strcpy (Equip->Requete, "Bad command\n");
	Envoie_Secur_Requete(Equip,Equip->Requete,strlen(Equip->Requete),"IMAP",Equip->retour_envoie);
	if (Equip->retour_envoie!=NULL)
		return Equip->retour_envoie;

	Equip->TailleBanniere = sizeof (BANNIERE_IMAP);
	MALLOC(Equip->banniere,sizeof (char) * Equip->TailleBanniere);
	ERROR_if (Equip->banniere == NULL);
	strcpy (Equip->banniere, BANNIERE_IMAP);

	while ((!Equip->fin) && (Equip->Tentative!=50) && (Nb_Error_Consecutive<3) && (attendre_Donnee (Equip->IdSocket, Time_Out_Service))) {
		Equip->NbOctet = recvfrom (Equip->IdSocket, &Equip->Requete, (sizeof(char)*TAILLE_REQUETE_SERVICE) - 1, 0, NULL, 0);
		if (Equip->NbOctet < 1){
			debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "NbOctet == %d   %s  Socket (%d) Erreur Consécutive (%d)",Equip->NbOctet, strerror (errno),Equip->IdSocket,Nb_Error_Consecutive);
			if (Equip->NbOctet == -1){ 
				if (!(errno == EINTR || errno == EAGAIN)){
					if (close_control (Equip->IdSocket) < 0)
						debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
					MYFREE(Equip->banniere); 
					return g_strdup ("Error IMAP :  NbOctet == -1");
				}
				sleep(1);
			}
			else
				sleep(1);
			Nb_Error_Consecutive++;
		}
		else {
			Nouveau_Octet_Recu (Equip->NbOctet);
			Equip->Requete[Equip->NbOctet] = 0; 
		
		
			Equip->i = 0;//Equip->NbOctet;
			while ((Equip->i<Equip->NbOctet) && (Equip->Requete[Equip->i]!=0))
				Equip->i++;			
			if (Equip->i!=Equip->NbOctet)
				debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Caractère final %d %d",Equip->NbOctet,Equip->i);
		
			if ((Equip->Finale = strstr(Equip->Requete,"\r\n"))==NULL){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Repere pas trouvé : ajout %d Octect",Equip->NbOctet);
					REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + Equip->NbOctet)));
					strcat (Equip->banniere, Equip->Requete);
					Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
			}
			else {
				Equip->nbLigne++;
				if ((Equip->Finale2 = strstr(Equip->Finale+2,"\r\n"))!=NULL){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "2 lignes en un coups");
					Equip->nbLigne++;
					Equip->Finale = Equip->Finale2;
				}
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Repere trouvé : lignes %d",Equip->nbLigne);
				Equip->NbOctet = Equip->Finale - Equip->Requete + 2;
				REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + Equip->NbOctet)));
				Equip->Requete[Equip->NbOctet] = 0;
				strcat (Equip->banniere, Equip->Requete);
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Bannière OK : %s\n",Equip->banniere);
				Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
				if (Equip->nbLigne==2){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin des donnees a recuperer (IMAP)");
					Equip->fin = TRUE;
				}
			}
			Nb_Error_Consecutive = 0;
		}
		Equip->Tentative++;
	}
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Bannière recuperer : %s\n",Equip->banniere);
	if (Equip->TailleBanniere != (strlen (Equip->banniere) + 1))
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur TailleBanniere (%d)!=(%d)",Equip->TailleBanniere,(strlen (Equip->banniere) + 1));
	if (close_control (Equip->IdSocket) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\n[%s] Socket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", Equip->Adresse_Ip_Char,strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	return Equip->banniere;
}

char     *Recupere_Banniere_SMTP (struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_SMTP  "SMTP : "
	gint Nb_Error_Consecutive;
	
	usleep (50000);	
	ERROR_if (Equip == NULL);
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion au serveur SMTP [%s]", Equip->Adresse_Ip_Char);
	if ((Equip->IdSocket = Connection_Equipement (Equip->Adresse_Ip_Char,NULL, PORT_SMTP, Time_Out_Connect_Service, 0,FALSE)) == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion refusée au serveur SMTP [%s]", Equip->Adresse_Ip_Char);
		return g_strdup ("Error SMTP: Connection refused");
	}

	Equip->chaine = NULL;
	Equip->banniere = NULL;
	Equip->Finale = NULL;
	Equip->Finale2 = NULL;
	Equip->retour_envoie = NULL;
	Equip->nbLigne = 0;
	Equip->fin = FALSE;
	Equip->Tentative = 0;
	Nb_Error_Consecutive = 0;

	strcpy (Equip->Requete, "helo\n");
	Envoie_Secur_Requete(Equip,Equip->Requete,strlen(Equip->Requete),"SMTP",Equip->retour_envoie);
	if (Equip->retour_envoie!=NULL)
		return Equip->retour_envoie;

	Equip->TailleBanniere = sizeof (BANNIERE_SMTP);
	MALLOC(Equip->banniere,sizeof (char) * Equip->TailleBanniere);
	ERROR_if (Equip->banniere == NULL);
	strcpy (Equip->banniere, BANNIERE_SMTP);

	while ((!Equip->fin) && (Equip->Tentative!=50) && (Nb_Error_Consecutive<3) && (attendre_Donnee (Equip->IdSocket, Time_Out_Service))) {
		Equip->NbOctet = recvfrom (Equip->IdSocket, &Equip->Requete, (sizeof(char)*TAILLE_REQUETE_SERVICE) - 1, 0, NULL, 0);
		if (Equip->NbOctet < 1){
			debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "NbOctet == %d   %s  Socket (%d) Erreur Consécutive (%d)",Equip->NbOctet, strerror (errno),Equip->IdSocket,Nb_Error_Consecutive);
			if (Equip->NbOctet == -1){ 
				if (!(errno == EINTR || errno == EAGAIN)){
					if (close_control (Equip->IdSocket) < 0)
						debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
					MYFREE(Equip->banniere); 
					return g_strdup_printf ("Error SMTP :  NbOctet == -1");
				}
				sleep(1);
			}
			else
				sleep(1);
			Nb_Error_Consecutive++;
		}
		else {
			Nouveau_Octet_Recu (Equip->NbOctet);
			Equip->Requete[Equip->NbOctet] = 0; 
		
		
			Equip->i = 0;//Equip->NbOctet;
			while ((Equip->i<Equip->NbOctet) && (Equip->Requete[Equip->i]!=0))
				Equip->i++;			
			if (Equip->i!=Equip->NbOctet)
				debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Caractère final %d %d",Equip->NbOctet,Equip->i);
		
			if ((Equip->Finale = strstr(Equip->Requete,"\r\n"))==NULL){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Repere pas trouvé : ajout %d Octect",Equip->NbOctet);
					REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + Equip->NbOctet)));
					strcat (Equip->banniere, Equip->Requete);
					Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
			}
			else {
				Equip->nbLigne++;
				if ((Equip->Finale2 = strstr(Equip->Finale+2,"\r\n"))!=NULL){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "2 lignes en un coups");
					Equip->nbLigne++;
					Equip->Finale = Equip->Finale2;
				}
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Repere trouvé : lignes %d",Equip->nbLigne);
				Equip->NbOctet = Equip->Finale - Equip->Requete + 2;
				REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + Equip->NbOctet)));
				Equip->Requete[Equip->NbOctet] = 0;
				strcat (Equip->banniere, Equip->Requete);
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Bannière OK : %s\n",Equip->banniere);
				Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
				if (Equip->nbLigne==2){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin des donnees a recuperer (SMTP)");
					Equip->fin = TRUE;
				}
			}
			Nb_Error_Consecutive = 0;
		}
		Equip->Tentative++;
	}
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Bannière recuperer : %s\n",Equip->banniere);
	if (Equip->TailleBanniere != (strlen (Equip->banniere) + 1))
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur TailleBanniere (%d)!=(%d)",Equip->TailleBanniere,(strlen (Equip->banniere) + 1));
	if (close_control (Equip->IdSocket) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\n[%s] Socket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", Equip->Adresse_Ip_Char,strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	return Equip->banniere;
}


char     *Recupere_Banniere_SSH(struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_SSH  "SSH : "
	gint Nb_Error_Consecutive;
	
	usleep (50000);	
	ERROR_if (Equip == NULL);
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion au serveur SSH [%s]", Equip->Adresse_Ip_Char);
	if ((Equip->IdSocket = Connection_Equipement (Equip->Adresse_Ip_Char,NULL, PORT_SSH, Time_Out_Connect_Service, 0,FALSE)) == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion refusée au serveur SSH [%s]", Equip->Adresse_Ip_Char);
		return g_strdup ("Error SSH : Connection refused");
	}

	Equip->chaine = NULL;
	Equip->banniere = NULL;
	Equip->Finale = NULL;
	Equip->Finale2 = NULL;
	Equip->retour_envoie = NULL;
	Equip->nbLigne = 0;
	Equip->fin = FALSE;
	Equip->Tentative = 0;
	Nb_Error_Consecutive = 0;

	strcpy (Equip->Requete, "Help\n");
	Envoie_Secur_Requete(Equip,Equip->Requete,strlen(Equip->Requete),"SSH",Equip->retour_envoie);
	if (Equip->retour_envoie!=NULL)
		return Equip->retour_envoie;

	Equip->TailleBanniere = sizeof (BANNIERE_SSH);
	MALLOC(Equip->banniere,sizeof (char) * Equip->TailleBanniere);
	ERROR_if (Equip->banniere == NULL);
	strcpy (Equip->banniere, BANNIERE_SSH);

	while ((!Equip->fin) && (Equip->Tentative!=50) && (Nb_Error_Consecutive<3) && (attendre_Donnee (Equip->IdSocket, Time_Out_Service))) {
		Equip->NbOctet = recvfrom (Equip->IdSocket, &Equip->Requete, (sizeof(char)*TAILLE_REQUETE_SERVICE) - 1, 0, NULL, 0);
		if (Equip->NbOctet < 1){
			debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "NbOctet == %d   %s  Socket (%d) Erreur Consécutive (%d)",Equip->NbOctet, strerror (errno),Equip->IdSocket,Nb_Error_Consecutive);
			if (Equip->NbOctet == -1){ 
				if (!(errno == EINTR || errno == EAGAIN)){
					if (close_control (Equip->IdSocket) < 0)
						debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
					MYFREE(Equip->banniere); 
					return g_strdup_printf ("Error SSH :  NbOctet == -1");
				}
				sleep(1);
			}
			else
				sleep(1);
			Nb_Error_Consecutive++;
		}
		else {
			Nouveau_Octet_Recu (Equip->NbOctet);
			Equip->Requete[Equip->NbOctet] = 0; 
		
		
			Equip->i = 0;//Equip->NbOctet;
			while ((Equip->i<Equip->NbOctet) && (Equip->Requete[Equip->i]!=0))
				Equip->i++;			
			if (Equip->i!=Equip->NbOctet)
				debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Caractère final %d %d",Equip->NbOctet,Equip->i);
		
			if ((Equip->Finale = strstr(Equip->Requete,"\n"))==NULL){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Repere pas trouvé : ajout %d Octect",Equip->NbOctet);
					REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + Equip->NbOctet)));
					strcat (Equip->banniere, Equip->Requete);
					Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
			}
			else {
				Equip->nbLigne++;
				if ((Equip->Finale2 = strstr(Equip->Finale+1,"\n"))!=NULL){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "2 lignes en un coups");
					Equip->nbLigne++;
					Equip->Finale = Equip->Finale2;
				}
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Repere trouvé : lignes %d",Equip->nbLigne);
				Equip->NbOctet = Equip->Finale - Equip->Requete + 1;
				REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + Equip->NbOctet)));
				Equip->Requete[Equip->NbOctet] = 0;
				strcat (Equip->banniere, Equip->Requete);
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Bannière OK : %s\n",Equip->banniere);
				Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
				if (Equip->nbLigne==2){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin des donnees a recuperer (SSH)");
					Equip->fin = TRUE;
				}
			}
			Nb_Error_Consecutive = 0;
		}
		Equip->Tentative++;
	}
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Bannière recuperer : %s\n",Equip->banniere);
	if (Equip->TailleBanniere != (strlen (Equip->banniere) + 1))
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur TailleBanniere (%d)!=(%d)",Equip->TailleBanniere,(strlen (Equip->banniere) + 1));
	if (close_control (Equip->IdSocket) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\n[%s] Socket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", Equip->Adresse_Ip_Char,strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	return Equip->banniere;
}


char     *Recupere_Banniere_TELNET (struct Struct_Liste_Equipement *Equip)
{
#define   BANNIERE_TELNET  "TELNET : "
	gint Nb_Error_Consecutive;
	
	usleep (50000);	
	ERROR_if (Equip == NULL);
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion au serveur TELNET [%s]", Equip->Adresse_Ip_Char);
	if ((Equip->IdSocket = Connection_Equipement (Equip->Adresse_Ip_Char,NULL,PORT_TELNET, Time_Out_Connect_Service, 0,FALSE)) == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion refusée au serveur TELNET [%s]", Equip->Adresse_Ip_Char);
		return g_strdup ("Error TELNET: Connection refused");
	}

	Equip->chaine = NULL;
	Equip->banniere = NULL;
	Equip->Finale = NULL;
	Equip->Finale2 = NULL;
	Equip->retour_envoie = NULL;
	Equip->nbLigne = 0;
	Equip->fin = FALSE;
	Equip->Tentative = 0;
	Equip->Telnet_Recuperer = -1;
	Nb_Error_Consecutive = 0;

	Equip->TailleBanniere = sizeof (BANNIERE_TELNET);
	MALLOC(Equip->banniere,sizeof (char) * Equip->TailleBanniere);
	ERROR_if (Equip->banniere == NULL);
	strcpy (Equip->banniere, BANNIERE_TELNET);

	while ((Equip->Telnet_Recuperer == -1) && (Equip->Tentative!=50) && (Nb_Error_Consecutive<3) && (attendre_Donnee (Equip->IdSocket, Time_Out_Service) != FALSE)) {
		Equip->NbOctet = recvfrom (Equip->IdSocket, &Equip->DonneeBrut, ((sizeof(char)*TAILLE_REQUETE_SERVICE) - 1), 0, NULL, 0);
		if (Equip->NbOctet < 1){
			debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "NbOctet == %d   %s  Socket (%d) Erreur Consécutive (%d)",Equip->NbOctet, strerror (errno),Equip->IdSocket,Nb_Error_Consecutive);
			if (Equip->NbOctet == -1){ 
				if (!(errno == EINTR || errno == EAGAIN)){
					if (close_control (Equip->IdSocket) < 0)
						debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
					MYFREE(Equip->banniere); 
					return g_strdup ("Error TELNET :  NbOctet == -1");
				}
				sleep(1);
			}
			else
				sleep(1);
			Nb_Error_Consecutive++;
		}
		else {	
			Nouveau_Octet_Recu (Equip->NbOctet);
			if ((Equip->DonneeBrut[0] & 0xff) == 0xff) {
				Equip->i = 0;
				while ((Equip->i < (Equip->NbOctet / 3)) && ((Equip->DonneeBrut[(Equip->i * 3)] & 0xff) == 0xff)) {
					Equip->DonneeBrut[(Equip->i * 3) + 1] = 0xfc;
					Equip->i++;
				}
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "(%s)Taille retour negociation : [%d]  total:[%d]", Equip->Adresse_Ip_Char, (Equip->i * 3), Equip->NbOctet);
				if ((Equip->i * 3) < Equip->NbOctet) {
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "(%s)Des donnees après la negociation  taille(%d)", Equip->Adresse_Ip_Char, (Equip->NbOctet - (Equip->i * 3)));
					/*for (j = 0; j < NbOctet; j++) {
				 	* printf ("[%d]", DonneeBrut[j]);
				 	* }
				 	* printf ("\n"); */
				
				
				
				
				
				
				
					for (Equip->j = (Equip->i * 3); Equip->j < Equip->NbOctet; Equip->j++)
						if ((Equip->DonneeBrut[Equip->j] != 10) && (Equip->DonneeBrut[Equip->j] != 13) && ((Equip->DonneeBrut[Equip->j] < 31) || (Equip->DonneeBrut[Equip->j] > 126)))
							Equip->DonneeBrut[Equip->j] = '$';
					Equip->DonneeBrut[Equip->NbOctet] = 0;
					Equip->TailleBanniere = Equip->TailleBanniere + (Equip->NbOctet - (Equip->i * 3));
					REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + 1)));
					strcat (Equip->banniere, Equip->DonneeBrut + (Equip->i * 3));
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "(%s) %s [%d]  TailleBanniere[%d] ", Equip->Adresse_Ip_Char, Equip->banniere, strlen (Equip->banniere), Equip->TailleBanniere);
				}
				Envoie_Secur_Requete(Equip,Equip->DonneeBrut,(Equip->i * 3),"TELNET",Equip->retour_envoie);
				if (Equip->retour_envoie!=NULL)
					return Equip->retour_envoie; 
			}
			else {
				for (Equip->j = 0; Equip->j < Equip->NbOctet; Equip->j++)
					if ((Equip->DonneeBrut[Equip->j] != 10) && (Equip->DonneeBrut[Equip->j] != 13) && ((Equip->DonneeBrut[Equip->j] < 31) || (Equip->DonneeBrut[Equip->j] > 126)))
						Equip->DonneeBrut[Equip->j] = '$';
				Equip->DonneeBrut[Equip->NbOctet] = 0;
				Equip->TailleBanniere = Equip->TailleBanniere + Equip->NbOctet;
				REALLOC(Equip->banniere,(sizeof (char) * (Equip->TailleBanniere + 1)));
				strcat (Equip->banniere, Equip->DonneeBrut);
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Standard - (%s) %s [%d] TailleBanniere[%d] ", Equip->Adresse_Ip_Char, Equip->banniere, strlen (Equip->banniere), Equip->TailleBanniere);
			}
			Nb_Error_Consecutive = 0;
		}
		Equip->Tentative++;
	}
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Bannière recuperer : %s\n",Equip->banniere);
	if (Equip->TailleBanniere != (strlen (Equip->banniere) + 1))
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur TailleBanniere (%d)!=(%d)",Equip->TailleBanniere,(strlen (Equip->banniere) + 1));
	if (close_control (Equip->IdSocket) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	return Equip->banniere;
}
